#!/usr/bin/perl -w

use strict;
use Getopt::Long;

package Nagios;

my $STATUSFILE="/var/log/nagios/status.dat";
my $CONFFILE="/etc/nagios/wlcg.cfg";

sub setError($) {
	my $self = shift;
	my $answer = shift;
	
	$self->{ERROR} = "Nagios Parser ERROR: $answer\n";
	
	1;
}

sub clearError() {
	my $self = shift;
	$self->{ERROR} = "";
	1;
}

sub fetchConfHostData {
	my $self = shift;
	my $attrs = shift;
	my $attrsSearch;
	my $host;
	my $type;

	if (@$attrs) {
		$attrsSearch = "(" . join (")|(" , @$attrs) . ")";
		($attrsSearch =~ /host_name/) or $attrsSearch .= "|(host_name)";
	}

	unless(open (STD, $CONFFILE)) {
		$self->setError("Failed opening config file $CONFFILE.");
		return 0;
	}	

	while (my $line = <STD>) {
		if ($line =~ /define\s+host\s*\{/) {
			$host = {};
			$type = 1;
		} elsif ($type) {
			if ($line =~ /\s*\}/) {
				if (exists $host->{host_name}) {
					if (exists $self->{HOSTS}->{$host->{host_name}}) {
						foreach my $key (keys %{$host}) {
							$self->{HOSTS}->{$host->{host_name}}->{$key} = $host->{$key};
						}
					} else {
						$self->{HOSTS}->{$host->{host_name}} = {%{$host}};
					}	
				}
				$type = 0;
			} elsif ( ( ! defined $attrsSearch || $line =~ /$attrsSearch/) && $line =~ /^\s*(\w+)\s+(\S.+?\S)(\s*;\s*\S.+?\S)?\s*$/) {
				$host->{$1}=$2;
			}
                }
	}

	close (STD);

	$self->{HOSTS_INTERNAL} = {%{$self->{HOSTS}}};

	1;
}

sub fetchStatusHostData {
	my $self = shift;
	my $attrs = shift;
	my $attrsSearch;
	my $host;
	my $type;

	if (@$attrs) {
		$attrsSearch = "(" . join (")|(" , @$attrs) . ")";
		($attrsSearch =~ /host_name/) or $attrsSearch .= "|(host_name)";
	}
	
	unless(open (STD, $STATUSFILE)) {
		$self->setError("Failed opening status file $STATUSFILE.");
		return 0;
	}

	while (my $line = <STD>) {
		if ($line =~ /host\s*\{/) {
			$host = {};
			$type = 1;
		} elsif ($type) { 
			if ($line =~ /\s+\}/) {
				if (exists $self->{HOSTS}->{$host->{host_name}}) {
					foreach my $key (keys %{$host}) {
						$self->{HOSTS}->{$host->{host_name}}->{$key} = $host->{$key};
					}
				} else {
					$self->{HOSTS}->{$host->{host_name}} = {%{$host}};
				}
				$type = 0;
			} elsif (( ! defined $attrsSearch || $line =~ /$attrsSearch/) && $line =~ /^\s+(\w+)=(\S.*?\S?)\s*$/) {
				$host->{$1}=$2;
			}
		}
	}
	close (STD);

	$self->{HOSTS_INTERNAL} = {%{$self->{HOSTS}}};
	
	1;
}

sub fetchStatusServiceData {
	my $self = shift;
	my $attrs = shift;
	my $attrsSearch;
	my $service;
	my $type;

	if (@$attrs) {
		$attrsSearch = "(" . join (")|(" , @$attrs) . ")";
		($attrsSearch =~ /host_name/) or $attrsSearch .= "|(host_name)";
		($attrsSearch =~ /service_description/) or $attrsSearch .= "|(service_description)";
	}

	unless(open (STD, $STATUSFILE)){
		$self->setError("Failed opening status file $STATUSFILE.");
		return 0;
	}

	while (my $line = <STD>) {
		if ($line =~ /service\s+\{/) {
			$service = {};
			$type = 1;
		} elsif ($type) {
			if ($line =~ /\s*\}/) {
				$service->{service_name} = lc($service->{service_description});
				$service->{service_name} =~ s/\s/_/g;

				$self->{HOSTS}->{$service->{host_name}} = {} unless (exists $self->{HOSTS}->{$service->{host_name}});
				$self->{HOSTS}->{$service->{host_name}}->{services} = {} unless (exists $self->{HOSTS}->{$service->{host_name}}->{services});

				if (exists $self->{HOSTS}->{$service->{host_name}}->{services}->{$service->{service_name}}) {
					foreach my $key (keys %{$service}) {
						$self->{HOSTS}->{$service->{host_name}}->{services}->{$service->{service_name}}->{$key} = $service->{$key};
					}
				} else {
					$self->{HOSTS}->{$service->{host_name}}->{services}->{$service->{service_name}} = {%{$service}};
				}
				$type = 0;
			} elsif (( ! defined $attrsSearch || $line =~ /$attrsSearch/) && $line =~ /^\s+(\w+)=(\S.*?\S?)\s*$/) {
				$service->{$1}=$2;
			}
		}
	}
	close (STD);
	
	$self->{HOSTS_INTERNAL} = {%{$self->{HOSTS}}};

	1;
}

sub fetchConfServiceData {
    my $self = shift;
    my $attrs = shift;
    my $attrsSearch;
    my $service;
    my $type;

    if (@$attrs) {
        $attrsSearch = "(" . join (")|(" , @$attrs) . ")";
        ($attrsSearch =~ /host_name/) or $attrsSearch .= "|(host_name)";
        ($attrsSearch =~ /service_description/) or $attrsSearch .= "|(service_description)";
    }

    unless(open (STD, $CONFFILE)) {
        $self->setError("Failed opening config file $CONFFILE.");
        return 0;
    }   

    while (my $line = <STD>) {
        if ($line =~ /define\s+service\s*\{/) {
            $service = {};
            $type = 1;
        } elsif ($type) {
            if ($line =~ /\s*\}/) {
                if (exists $service->{service_description} && exists $service->{host_name}) {
                    $service->{service_name} = lc($service->{service_description});
                    $service->{service_name} =~ s/\s/_/g;
        
                    $self->{HOSTS}->{$service->{host_name}} = {} unless (exists $self->{HOSTS}->{$service->{host_name}});
                    $self->{HOSTS}->{$service->{host_name}}->{services} = {} unless (exists $self->{HOSTS}->{$service->{host_name}}->{services});
        
                    if (exists $self->{HOSTS}->{$service->{host_name}}->{services}->{$service->{service_name}}) {
                        foreach my $key (keys %{$service}) {
                            $self->{HOSTS}->{$service->{host_name}}->{services}->{$service->{service_name}}->{$key} = $service->{$key};
                        }
                    } else {
                        $self->{HOSTS}->{$service->{host_name}}->{services}->{$service->{service_name}} = {%{$service}};
                    }
                    $type = 0;
                }
            } elsif ( ( ! defined $attrsSearch || $line =~ /$attrsSearch/) && $line =~ /^\s*(\w+)\s+(\S.+?\S)(\s*;\s*\S.+?\S)?\s*$/) {
                $service->{$1}=$2;
            }
        }
    }

    close (STD);

    $self->{HOSTS_INTERNAL} = {%{$self->{HOSTS}}};

    1;
}


sub filterHosts($) {
	my $self = shift;
	my $filter = shift;
	my @delete;

	$self->{FILTER} = 1;

	unless (%{$self->{HOSTS}}) {
		$self->setError("List of hosts must be populated first.");
		return 0;
	}

	foreach my $host ( values %{$self->{HOSTS}} ) {
		my $test = 0;

		foreach my $key ( keys %{$filter} ) {
			exists $host->{$key} or next;
			if ($filter->{$key} =~ /^\-?\d+/) {
				unless ($host->{$key} == $filter->{$key}) {
					$test = 1;
					last;
				}
			} else {
				unless ($host->{$key} =~ /$filter->{$key}/) {
					$test = 1;
					last;
				}
			}
		}
		
		$test and push @delete, $host->{host_name};
	}

	foreach (@delete) {
		delete $self->{HOSTS}->{$_};
	}

	1;
}

sub filterHostsByService($) {
	my $self = shift;
	my $filter = shift;
	my @delete;

	$self->{FILTER} = 1;
	
	unless (exists $filter->{service_description}) {
		$self->setError("Service filter must contain service_description.");
		return 0;
	} 

	unless (%{$self->{HOSTS}}) {
		$self->setError("List of hosts must be populated first.");
		return 0;
	}

	foreach my $host ( values %{$self->{HOSTS}} ) {
		my $test = 0;

		exists $host->{services} or next;

		foreach my $service ( values %{$host->{services}} ) {
			if ($service->{service_description} =~ /$filter->{service_description}/) {
				foreach my $key ( keys %{$filter} ) {
					exists $service->{$key} or next;
					if ($filter->{$key} =~ /\d+/) {
						unless ($service->{$key} == $filter->{$key}) {
							$test = 1;
							last;
						}
					} else {
						unless ($service->{$key} =~ /$filter->{$key}/) {
							$test = 1;
							last;
						}
					}
				}
			}
			$test and last;
		}

		$test and push @delete, $host->{host_name};
	}

	foreach (@delete) {
		delete $self->{HOSTS}->{$_};
	}

	1;
}

sub new {
	my $class = shift;
	my $self  = {};
	$self->{HOSTS} = {};
	$self->{HOSTS_INTERNAL} = {};
	$self->{FILTER} = 0;
	$self->{ERROR} = "";
	bless ($self, $class);
	$self;
}

sub hosts {
	my $self = {};
	return $self->{HOSTS};
}

sub clearFilter {
	my $self = {};
	$self->{HOSTS} = {%{$self->{HOSTS}}} if ($self->{FILTER});
	return 1;
}

1;
